<?php

namespace Drupal\Tests\widget_provider_example\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * This class provides methods specifically for testing UI Elements of the WidgetProviderApi Examples.
 *
 * @group widget_provider_example
 */
class WidgetExampleFunctionalTest extends BrowserTestBase {
  use \Drupal\Tests\node\Traits\NodeCreationTrait;

   /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'user',
    'node',
    'views',
    'widget_provider_api',
    'widget_provider_example',
  ];

  /**
   * Plain theme with no styling for better testing.
   *
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stable';

  /**
   * Permissions for the admin user.
   *
   * @var array
   */
  protected $adminPermissions = [
    'access widgets',
  ];

  /**
   * User with proper permissions for module configuration.
   *
   * @var \Drupal\user\Entity\User|false
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Gibt error, aber sonst Container error.
    parent::setUp();
    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser($this->adminPermissions);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * {@inheritdoc}
   */
  protected function tearDown(): void {
    parent::tearDown();
  }

  /**
   * Tests to see if the Widget View Example url exists.
   */
  public function testURLViewExampleExists() {
    $this->drupalGet('/widget/example-provider1/widget-view-example');
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests to see if the View is rendered on the site.
   */
  public function testViewExists(){
    $this->drupalGet('/widget/example-provider1/widget-view-example');
    $this->assertSession()->pageTextContains('No front page content has been created yet.');
  }

  /**
   * Tests to see if the Widget Node Example url exists.
   */
  public function testURLNodeExampleExists() {
    // Create an example node:
    $node = $this->createNode();
    $this->drupalGet('/widget/example-provider1/widget-node-example', ['query' => ['nid' => (string) $node->id(), 'kid' => '1']]);
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests to see if the Node is rendered.
   */
  public function testNodeExists(){
    // Create an example node:
    $node = $this->createNode();
    $this->drupalGet('/widget/example-provider1/widget-node-example', ['query' => ['nid' => (string) $node->id(), 'kid' => '1']]);
    $this->assertSession()->pageTextContains('Hello world!');
  }

  /**
   * Tests to see if the Widget IFrame Example url exists.
   *
   * @return void
   */
  public function testURLIframeExampleExists() {
    $this->drupalGet('/widget/example-provider1/widget-iframe-example');
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests to see if the iframe is rendered.
   */
  public function testIframeExists(){
    $this->drupalGet('/widget/example-provider1/widget-iframe-example');
    $this->assertSession()->elementAttributeExists('css', 'iframe', 'src');
    $this->assertSession()->elementAttributeContains('css', 'iframe', 'src', 'https://www.google.com/');
  }

  /**
   * Tests to see if the Widget Js Example url exists.
   */
  public function testURLJsExampleExists() {
    $this->drupalGet('/widget/example-provider2/widget-js-example', ['query' => ['kid' => '1', 'hid' => '1']]);
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests to see if the Json is rendered.
   */
  public function testJsExists() {
    $this->drupalGet('/widget/example-provider2/widget-js-example', ['query' => ['kid' => '1', 'hid' => '1']]);
    $this->assertSession()->responseContains('This is an example from JS Example');
  }

  /**
   * Tests to see if there is no access, without the required parameter.
   *
   * @return void
   */
  public function testJsRequiredAccess() {
    $this->drupalGet('/widget/example-provider2/widget-js-example', ['query' => ['hid' => '1']]);
    $this->assertSession()->statusCodeEquals(404);
  }

  /**
   * Tests to see if there is access, even without the optional Parameter.
   */
  public function testJsOptionalAccess() {
    $this->drupalGet('/widget/example-provider2/widget-js-example', ['query' => ['kid' => '1']]);
    $this->assertSession()->responseContains('This is an example from JS Example');
  }

  /**
   * Tests if some random Widget doesn't exist.
   *
   * @return void
   */
  public function testWidgetDoesNotExist(){
    $this->drupalGet('/widget/example-provider2/randomWidget');
    $this->assertSession()->statusCodeEquals(404);
  }

  /**
   * Tests to see if there is access to the widget page if logged out.
   */
  public function testAccessLoggedOut() {
    $this->drupalLogout();
    $this->drupalGet('/widget/example-provider1/widget-iframe-example');
    $this->assertSession()->statusCodeEquals(403);
  }

}
