<?php

namespace Drupal\widget_provider_api\Widget;

use Drupal\Core\Render\BareHtmlPageRenderer;
use Drupal\views\Plugin\views\field\Boolean;
use Drupal\widget_provider_api\Exceptions\WidgetValidationUnallowedReferrerException;
use Drupal\widget_provider_api\Exceptions\WidgetValidationParametersException;
use Drupal\widget_provider_api\Exceptions\WidgetValidationWrongChecksumException;
use Drupal\widget_provider_api\Exceptions\NoRegisteredWidgetParameterException;
use Drupal\widget_provider_api\Exceptions\ChecksumSecretWrongFormatException;


abstract class WidgetBase implements WidgetInterface {

  /**
   * The Widget ID.
   *
   * @var string
   */
  private $id;

  /**
   * The Widget Specific required Parameters.
   *
   * @var array
   */
  private array $requiredParameters = [];

  /**
   * The Widget Specific allowed Referrers.
   *
   * @var array
   */
  private array $allowedReferrers = [];

  /**
   * A list of all optional parameters.
   *
   * @var array
   */
  private array $optionalParameters = [];

  /**
   * A checksum to verify the User.
   *
   * @var Boolean|String
   */
  private $validateChecksumSalt;

  /**
   * Factory method Constructor.
   *
   * @param array $optionalParameters
   * @return WidgetInterface
   */
  public static function create($id, array $requiredParameters = [], array $optionalParameters = [], array $allowedReferrers = [], $validateChecksumSalt = FALSE): WidgetInterface {
    return new static($id, $optionalParameters, $requiredParameters, $allowedReferrers, $validateChecksumSalt);
  }

  /**
   * Constructor.
   *
   * @param string $id
   * @param array $requiredParameters
   * @param array $optionalParameters
   * @param array $allowedReferrers
   */
  public function __construct($id, array $requiredParameters = [], array $optionalParameters = [], array $allowedReferrers = [], $validateChecksumSalt = FALSE) {
    $this->id = $id;
    $this->requiredParameters = $requiredParameters;
    $this->optionalParameters = $optionalParameters;
    $this->allowedReferrers = $allowedReferrers;
    $this->validateChecksumSalt = $validateChecksumSalt;
  }

  /**
   * Build the Response.
   * Return false to throw a NotFoundHttpException exception (404).
   *
   * @return array|Response|false
   */
  public function build() {
      $buildContent = $this->buildContent();
      if (is_array($buildContent)) {
        // The result was a Drupal render array, we have to render it, otherwise
        // it would be output in site context (template):
        // TODO: LATER think about how we could inject the response here and how
        // to configure the expected response type (with or without theme...)
        // @see https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Render%21HtmlResponse.php/class/HtmlResponse/8.2.x

        // Rander in bare page without template around:
        $attachments = \Drupal::service('html_response.attachments_processor');
        $renderer = \Drupal::service('renderer');
        $bareHtmlPageRenderer = new BareHtmlPageRenderer($renderer, $attachments);
        //  @see https://api.drupal.org/api/drupal/core%21lib%21Drupal%21Core%21Render%21BareHtmlPageRendererInterface.php/interface/BareHtmlPageRendererInterface/8.2.x
        $response = $bareHtmlPageRenderer->renderBarePage($buildContent, $this->getId(), 'markup');
        // TODO: This still contains CSS from theme. We have to decide what we really want here and how to control:
        // ALTERNATIVE WITHOUT ANY WRAPPING HTML:
        // $response = new HtmlResponse();
        // // Render RenderArray as response:
        // $response->setContent($buildContent);

        return $response;
      } else {
        return $buildContent;
      }
    //}
    throw new WidgetValidationUnallowedReferrerException('You are not allowed to see this Widget!');
  }

  /**
   * Validates all important Parameters
   *
   * @return Boolean
   */
  public function validateAll(){
    if(!$this->validateAllowedReferer()){
      throw new WidgetValidationUnallowedReferrerException();
    }
    if(!$this->validateChecksum()){
      throw new WidgetValidationWrongChecksumException();
    }
    if(!$this->validateParameters()){
      throw new WidgetValidationParametersException();
    }
    return true;
  }

  /**
   * Validates the Widget Parameters.
   * Checks if all required parameters are present.
   *
   * @return Boolean
   */
  public function validateParameters() {
    $request = \Drupal::request();
    $requiredParameters = $this->getRequiredParameters();
    if (!empty($requiredParameters)) {
      foreach ($requiredParameters as $requiredParameterKey) {
        if (!$request->query->has($requiredParameterKey)) {
          // A required parameter is missing:
          return false;
        }
      }
    }
    // All required parameters are present!
    return true;
  }

  /**
   * Validates the Widget Checksum.
   *
   * @return Boolean
   */
  public function validateChecksum() {
    $salt = $this->getValidateChecksumSecret();
    if ($salt === FALSE) {
      return TRUE;
    }
    $request = \Drupal::request();
    $queryParameters = $this->getAllParameters();
    $queryParameters['ts'] = $request->server->get('REQUEST_TIME');
    if (empty($salt) || $salt === TRUE) {
      throw new ChecksumSecretWrongFormatException();
    }
    ksort($queryParameters);
    $queryParametersSalted  = ($salt . implode('', $queryParameters));
    $hash = hash('sha256', $queryParametersSalted);
    if (($request->query->get('s')) == $hash) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Validates the Allowed Referrers of the widget.
   *
   * @param String $referrerTld
   * @return Boolean
   */
  public function validateAllowedReferer(): bool {
    if (empty($this->allowedReferrers)) {
      return true;
    }
    $request = \Drupal::request();
    $referrerTld = $request->headers->get('referer');
    if(empty($referrerTld)){
      return true;
    }
    return in_array($referrerTld, $this->allowedReferrers);
  }

  /**
   * Retrieve the Widget id.
   *
   * @return String
   */
  public function getId(): string {
    return $this->id;
  }

  /**
   * Retrieve all Allowed Widget Referrers.
   *
   * @return Array
   */
  public function getAllowedReferrers(): array {
    return $this->allowedReferrers;
  }

  /**
   * Retrieve required Parameters.
   *
   * @return Array
   */
  public function getRequiredParameters(): array {
    if (empty($this->requiredParameters)) {
      throw new \Exception('The widget "' . $this->id . '" has no required Parameters!');
    }
    return $this->requiredParameters;
  }

  /**
   * Get all optional Parameters
   *
   * @return Array
   */
  public function getOptionalParameters(): array {
    if (empty($this->optionalParameters)) {
      return [];
    }
    return $this->optionalParameters;
  }

  /**
   * Retrieve the validateChecksumSecret
   */
  public function getValidateChecksumSecret(){
    return $this->validateChecksumSecret;
  }

  /**
   * Retrieve all Parameters.
   *
   * @return Array
   */
  public function getAllParameters(): array {
    $requiredParas = $this->getRequiredParameters();
    $optionalParas = $this->getOptionalParameters();
    return array_merge($requiredParas, $optionalParas);
  }

  /**
   * Returns the value of the given registered $parameterKey. $default if empty.
   *
   * @param string $parameterKey
   * @param mixed $default The default if the parameter does not exist.
   * @throws NoRegisteredWidgetParameterException
   * @return mixed
   */
  public function getParameterValue($parameterKey, $default = null) {
    if (!in_array($parameterKey, $this->getAllParameters())) {
      throw new NoRegisteredWidgetParameterException($parameterKey . ' is not a registered widget parameter. You can only retrieve registered required or optional parameters. Did you forget to list them in this widget?');
    }
    $request = \Drupal::request();
    return $request->query->get($parameterKey, $default);
  }
}
