<?php

namespace Drupal\widget_provider_api\Widget;

use Drupal\widget_provider_api\Exceptions\WidgetNotFoundException;

abstract class WidgetProviderBase implements WidgetProviderInterface {

  /**
   * The Widget Provider ID.
   *
   * @var string
   */
  private $id;

  /**
   * Widgets of the provider.
   *
   * @var WidgetInterface[]
   */
  private array $widgets = [];

  /**
   * Static Factory Constructor.
   *
   * @param string $id
   * @param WidgetInterface[] $widgets
   * @return WidgetProviderInterface
   */
  public static function create(string $id, array $widgets): WidgetProviderInterface {
    return new static($id, $widgets);
  }

  /**
   * Constructor.
   *
   * @param string $id
   * @param WidgetInterface[] $widgets
   */
  public function __construct(string $id, array $widgets) {
    $this->id = $id;
    $this->addWidgets($widgets);
  }

  /**
   * Adds a widget to the list of widgets (indexed by its ID).
   *
   * @param WidgetInterface $widget
   * @return WidgetProviderInterface
   */
  protected function addWidget(WidgetInterface $widget): WidgetProviderInterface {
    $widgetId = $widget->getId();
    strtolower($widgetId);
    if (preg_match('/[^a-z_\-0-9]/i', $widgetId)) {
      throw new \Exception('The Name of "' . $widgetId . '" is invalid, the following Characters: "/[^a-z_\-0-9]/i" are not allowed');
    }
    if (isset($this->widgets[$widgetId])) {
      throw new \Exception('A widget with the ID "' . $widgetId . '" is already registered.');
    }
    $this->widgets[$widgetId] = $widget;
    return $this;
  }

  /**
   * Adds widget providers to the list of widget providers.
   *
   * @param WidgetInterface[] $widgets
   * @return WidgetProviderInterface
   */
  protected function addWidgets(array $widgets): WidgetProviderInterface {
    foreach ($widgets as $key => $widget) {
      $this->addWidget($widget);
    }
    return $this;
  }

  /**
   * Returns all available widgets.
   *
   * @return WidgetInterface[]
   */
  public function getWidgets(): array {
    return $this->widgets;
  }

  /**
   * Get Widget Object.
   *
   * @param string $widgetId
   * @return WidgetInterface
   */
  public function getWidget($widgetId): WidgetInterface {
    if (!empty($this->widgets[$widgetId])) {
      return $this->widgets[$widgetId];
    } else {
      throw new WidgetNotFoundException($widgetId . ' does not exist.');  //TODO: Muss noch gecatched werden bei Benutzung
    }
  }

  /**
   * Get the Widget Provider ID.
   *
   * @return  string
   */
  public function getId(): string {
    return $this->id;
  }
}
